
#include "Arduino.h"

char inputPin = 5; // entrée T1 pour Timer1

volatile uint16_t count_high,count_low;
volatile uint32_t count;
volatile uint16_t ni; // compteur d'interruptions
uint16_t ninter = 60; // nombre d'interruptions pour un comptage 
float deltaT;
                       
void start_count() { 
  // Timer 2 : génération d'interruptions périodiques
  TCCR2A |= (1 << WGM21); // CTC mode 'clear timer on compare), top = OCR2A
  OCR2A = 0xFF; // fréquence d'interruption 16 MHz /1024/256
  TIMSK2 = 1 << TOIE2; // overflow interrupt enable
  TCNT2 = 0;
  ni = 0;
  
  // Timer 1 : compteur d'impulsions
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  TIMSK1 = 1<<TOIE1; // overflow interrupt enable
  count = 0;
  
  sei(); // activation des interruptions
  TCCR2B |= (1 << CS12) | (1 << CS11) | (1 << CS10); // prescaler = 1024
  TCCR1B |= (1 << CS12) | (1 << CS11) | (1 << CS10); // external clock on rising edge
}
                        
ISR(TIMER2_OVF_vect) { // Overflow interrupt
   ni++;
   if (ni==ninter) {
     ni = 0;
     count_low = TCNT1;
     TCNT1 = 0;
     count = ((uint32_t)count_high)<<16 | count_low;
    count_high = 0;
   }
}
                        
ISR(TIMER1_OVF_vect) {
    count_high++;
}
                  
void setup() {
  Serial.begin(115200);
  pinMode(inputPin,INPUT);
  deltaT = 1.0*ninter*1024*256/F_CPU;
  start_count();
}
                         
void loop() {
  delay(1000);
  float freq = count/deltaT;
  float prec = 1.0/deltaT;
  Serial.print("Frequence (Hz) = ");
  Serial.println(freq,DEC);
  Serial.print("Precision (Hz) = ");
  Serial.println(prec,DEC);
  
}
                          